package org.slub.rosetta.dps.repository.plugin.storage.nfs;

import com.exlibris.core.sdk.storage.containers.StoredEntityMetaData;
import com.exlibris.digitool.common.dnx.DnxDocument;
import com.exlibris.digitool.common.dnx.DnxDocumentFactory;
import com.exlibris.digitool.common.storage.Fixity;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.junit.runners.JUnit4;
import org.springframework.beans.factory.BeanInitializationException;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import static java.nio.file.Path.of;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

/**
 * Tests for {@link org.slub.rosetta.dps.repository.plugin.storage.nfs.SLUBStoragePlugin}.
 *
 * @author andreas.romeyke@slub-dresden.de (Andreas Romeyke)
 */
@RunWith(JUnit4.class)

public class TestSLUBStoragePlugin {
  private static SLUBStoragePlugin mock;
  private static Path testroot;
  private static Path foopath;
  @Before
  public void setUp() throws IOException {
    testroot = Files.createTempDirectory("testslubstorage");
    foopath = of(testroot.toAbsolutePath() + "/foo");
    Files.createFile(foopath);
    mock = new SLUBStoragePlugin();
    HashMap<String, String> map = new HashMap<String,String> ();
    map.put("DIR_ROOT", testroot.toAbsolutePath().toString());
    map.put("BLOCK_SIZE", "8192");
    map.put("FILES_HANDLING_METHOD", "move");
    mock.setParameters( map );
    mock.init(map);
  }
//  @Test
//  public  void getDirRoot() {
    //SLUBStoragePlugin mock = new SLUBStoragePlugin();
    //assertEquals(mock.getDirRoot());
//  }

  @Test
  public void getStreamDirectory() {
    File calced_path = mock.getStreamDirectory("foo", "bar");
    assertEquals("getStreamDirectory test", "bar", calced_path.getName());
  }
  @Test
  public void checkFixity() {
    List<Fixity> fixities = new ArrayList<>();
    try {
      var result = mock.checkFixity(fixities, foopath.toString());
      assertTrue("checkFixity(.., "+foopath.toString()+")", result);
    }
    catch (Exception e) {
      assertEquals("checkFixity", "foo", e.getMessage());
    }
  }

  @Test
  public void getFullFilePath() {
    assertEquals( "getFullFilePath", testroot.toAbsolutePath()+"/2022", mock.getFullFilePath("/2022"));
  }
  @Test
  public void retrieveEntity() {
    try {
      InputStream is = mock.retrieveEntity("/foo");
      assertTrue("retrieveEntity", true);
    } catch (Exception e) {
      assertEquals("retrieveEntity", "", e.getMessage());
    }
  }
  @Test
  public void retrieveEntity_dirroot() {
    try {
      InputStream is = mock.retrieveEntity(testroot.toAbsolutePath() + "/foo");
      assertTrue("retrieveEntity", true);
    } catch (IllegalArgumentException e) {
      assertEquals("retrieveEntity", "", e.getMessage());
    } catch (IOException e) {
      assertTrue("retrieveEntity", e.getMessage().contains("should be relative, but starts with ROOT"));
    }
  }
  @Test
  public void retrieveEntitybyRange() {
    try {
      byte[] bytes = mock.retrieveEntityByRange("/foo", 0, 0);
      assertTrue("retrieveEntitybyRange", true);
    } catch (java.io.EOFException e) {
      assertEquals("retrieveEntitybyRange", null, e.getMessage());
    } catch (Exception e) {
      assertEquals("retrieveEntitybyRange", "", e.getMessage());
    }
  }

  @Test
  public void storeEntity() throws IOException {
    byte[] bytes = "teststring".getBytes(StandardCharsets.UTF_8);
    InputStream is = new ByteArrayInputStream(bytes);
    StoredEntityMetaData sem = new StoredEntityMetaData();
    sem.setCurrentFilePath("/bar");
    sem.setIePid(null);
    File file = new File("resources/iemets.xml");
    InputStream ioStream = new FileInputStream(file);
    InputStreamReader isr = new InputStreamReader(ioStream);
    char[] charArray = new char[(int)file.length()];
    isr.read(charArray);
    String iemets = String.valueOf(charArray);
    try {
      DnxDocument dnx = DnxDocumentFactory.getInstance().createDnxDocument(false);
      dnx.setDocumentXml(iemets);
      sem.setIeDnx(dnx);
      sem.setEntityType(StoredEntityMetaData.EntityType.IE);
      mock.storeEntity(is, sem);
    }
    catch (NoClassDefFoundError e) {
      /* FIXME: Test irgendwie den org/springframework/context/ApplicationContext verpassen... */
      assertEquals("storeEntity", "org/springframework/context/ApplicationContext", e.getMessage());
    }
    catch (BeanInitializationException e) {
      //assertEquals("storeEntity2", "Could not load properties; nested exception is java.io.FileNotFoundException: class path resource [global.properties] cannot be opened because it does not exist", e.getMessage());
    }
  }
  /*
  @Test
  public void getStreamRelativePath() throws Exception {
    SLUBStoragePlugin mock = new SLUBStoragePlugin();
    HashMap <String, String> map = new HashMap<String,String> ();
    map.put("DIR_ROOT", "./");
    map.put("DIR_PREFIX", "./");
    map.put("FILES_HANDLING_METHOD", "move");
    mock.setParameters( map );

    // TODO:
    StoredEntityMetaData storedEntityMetaData = new StoredEntityMetaData();
    File file = new File("resources/iemets.xml");
    InputStream ioStream = new FileInputStream(file);
    InputStreamReader isr = new InputStreamReader(ioStream);
    char[] charArray = new char[(int)file.length()];
    isr.read(charArray);
    String iemets = new String(charArray);
    System.out.println(iemets);
    DnxDocument iedoc = DnxDocumentFactory.getInstance().parse(iemets);
    storedEntityMetaData.setIeDnx(iedoc);
    storedEntityMetaData.setEntityType(StoredEntityMetaData.EntityType.FILE); //search in File-AMD
    storedEntityMetaData.setCurrentFilePath("foo");
    storedEntityMetaData.setEntityPid("FL225130");
    storedEntityMetaData.setIePid("IE225095");
    mock.getStreamRelativePath( storedEntityMetaData);
  }
  */


}
